#include "mainwindow.h"
#include "ui_mainwindow.h"
#include "caritem.h"
#include "trafficlog.h"

#include <QFileDialog>
#include <QSequentialAnimationGroup>
#include <QPropertyAnimation>

// Create a mapping from names of cars to their on screen representation.
QMap<QString, CarItem*> newCarMap (TrafficLog log) {
    QMap<QString, TrafficLogItem> map;
    for (TrafficLogItem item : log) {
        if (map.contains(item.name())) {
            if (item.time() < map[item.name()].time()) {
                map[item.name()] = item;
            }
        } else {
            map.insert(item.name(), item);
        }
    }

    QMap<QString, CarItem*> carMap;
    for (auto it = map.constBegin(); it != map.constEnd(); ++it) {
        TrafficLogItem logitem = it.value();
        carMap.insert(it.key(), new CarItem (logitem.pos()));
    }

    return carMap;
}

// Update the scene with the animation derived from the log file.
void MainWindow::updateSceneWithLog(TrafficLog log) {
    auto carMap = newCarMap(log);

    for (auto it : carMap) {
        scene.addItem (it);
    }

    auto animation = log.animation(carMap);
    animation->start();
}

// Add the roads from the map to the scene.
void MainWindow::updateSceneWithMap(trafficmap map) {
    map.addToScene(&scene);
}

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    scene(),
    ui(new Ui::MainWindow)
{
    ui->setupUi(this);
    ui->trafficView->show();
    ui->trafficView->setScene(&scene);
}

MainWindow::~MainWindow()
{

    delete ui;
}

// Load dialog for the log.
// Warning! doesn't do much checking on the validity of the file!
void MainWindow::loadLog() {
    const QString fileName = QFileDialog::getOpenFileName (this, "Open Log");
    QFile file(fileName);
    auto log = readTrafficLogFile (file);
    updateSceneWithLog (log);
}

// Load dialog for the map.
// Warning! (again) doesn't do much checking on the validity of the file!
void MainWindow::loadMap() {
    const QString fileName = QFileDialog::getOpenFileName (this, "Open Log");
    QFile file(fileName);
    auto map = map_from_file (file);
    updateSceneWithMap (map);
}

